// -------------------------------
// PART ONE: define common tools, constants
// -------------------------------
// used when testing in Chrome, rather than in MobileSafari
DEBUG = (window.location.href.indexOf('debug') > -1);

Q = function(s){
    return encodeURIComponent(s);
}

B = function(path) {
    // this is the magical hack that lets javascript send strings to Objective-C
    if (DEBUG) {
        //        alert(path);
        return;
    }
    window.location.href = 'bridge:///' + path;
}

var isAndroid = (navigator.userAgent.match(/Android/i));

REMOTE_LOG = function(s){
    $.ajax({
           url: '/logger/',
           type: 'POST',
           data: {'pgid': GAMESTATE.PGID, 's': s},
           });
};

// Getting Query Params
// http://stackoverflow.com/questions/901115/how-can-i-get-query-string-values-in-javascript
// Usage: $.QueryString["param"]
(function($) {
 $.QueryString = (function(a) {
                  if (a == "") return {};
                  var b = {};
                  for (var i = 0; i < a.length; ++i)
                  {
                  var p=a[i].split('=');
                  if (p.length != 2) continue;
                  b[p[0]] = decodeURIComponent(p[1].replace(/\+/g, " "));
                  }
                  return b;
                  })(window.location.search.substr(1).split('&'))
 })(jQuery);

READY_FOR_USER = false;
READY_TO_PLAY = false;
STORY_ID = $.QueryString["StoryID"];

IS_HOURLY = $.QueryString["style"] === '4hour';

// -------------------------------
// PART TWO: functions to data from Obj-C
// -------------------------------

// Local Notificactions
SCHEDULED = false;
schedule_local_notif = function(message, secondsFromNow){
    if (SCHEDULED) return;
    B('reminder?identifier=timegate&message=' + Q(message)
      + '&secondsFromNow=' + secondsFromNow);
    SCHEDULED = true;
};

did_reminder = function(){};

unschedule_local_notif = function(message, secondsFromNow){
    B('unschedulereminder?identifier=timegate');
};

did_unschedulereminder = function(){};

// Pulling data from Obj-C
do_render = function(){
    // "render" is a name from the client
    // similar to template rendering in Make or Django.
    // we use it here to get strings and counters from the user's gamestate

    // Make subquery for objectinfo
    var storyQuery = Q('{"identifier":"'+STORY_ID+'", "infoClass":"StoryInfo"}');
    var landingQuery = Q('{"identifier":"'+STORY_ID+'", "infoClass":"LandingInfo"}');

    B('render?'
      + 'SupportID=' + Q('[SupportID]')
      + '&PGID=' + Q('[PGID]')
      + '&ReferralCode=' + Q('[ReferralCode]')
      + '&ReferredFriends=' + Q('[NumFriends]')
      + '&EarnedReferralPasses=' + Q('[EarnedReferralPasses]')
      + '&objectinfo_story=' + storyQuery
      + '&objectinfo_landing=' + landingQuery
      );
};

did_render = function(results){
    GAMESTATE = {};
    GAMESTATE.StoryID = STORY_ID;
    GAMESTATE.SupportID = results.SupportID;
    GAMESTATE.PGID = results.PGID;
    GAMESTATE.ReferralCode = results.ReferralCode;
    GAMESTATE.ReferredFriends = results.ReferredFriends;
    GAMESTATE.EarnedReferralPasses = results.EarnedReferralPasses;
    GAMESTATE.StoryInfo = results.objectinfo_story;
    GAMESTATE.LandingInfo = results.objectinfo_landing;

    combine_app_and_server_info();
};

// -------------------------------
// PART FOUR: Make it work on iOS
// -------------------------------


// Hack to scale the screen for iPads
// since we don't have a responsive web-design yet!
if (!isAndroid && window.screen.width > 320){
    var scale = window.screen.width / 320.0;
    var val = "width=320px, initial-scale=" + scale + ", user-scalable=0";
    document.getElementById('viewport').setAttribute("content", val);
}

// On load, attach handlers to buttons
// Uses the touches API to be more responsive
$(function(){

  $('.btn').bind('touchstart', function(){
                 this.classList.add('hover');
                 });
  $('.btn').bind('touchend', function(){
                 this.classList.remove('hover');
                 CLICK_ON_ELEMENT(this.id);
                 });
  $('.btn').bind('click', function(){ CLICK_ON_ELEMENT(this.id); });

  // start the data load, but only if we are in ios-webview land
  if (DEBUG){
  LOG('faking it!');
  did_render({
             "PGID":"matt",
             "PASSES":"2",
             "SECONDS_TO_WAIT":"15",
             "StoryID":"love.life",
             "StoryName":"Hollywood Crush",
             "ChapterNum":"2",
             });
  } else {
  do_render();
  }

  // disable touches temporarily after loading a page
  // otherwise errant touch-ends seem to get fired on launch
  // when switching pages
  prevent_double_touch();
  });


TOUCHES_DISABLED = true;
prevent_double_touch = function(){
    TOUCHES_DISABLED = true;
    setTimeout(function(){
               TOUCHES_DISABLED = false;
               }, 1000);
}


window.onerror = function(msg, url, linenumber) {
    setTimeout(function(){
               var error = 'Error message: '+msg+'\nURL: '+url+'\nLine Number: '+linenumber;
               alert(error);
               REMOTE_LOG(error);
               }, 1);
    return true;
}

// Prevent scrolling
/*
 document.ontouchmove = function(event){
 event.preventDefault();
 }
 */


// -------------------------------
// PART FIVE: functions specific to this page
// -------------------------------

_COUNT = 0;
LOG = function(s){
    _COUNT +=1;
    $('#log').html($('#log').html() + '<br>' + _COUNT + ' ' + s);
    console.log(s);
    //B("logger?tag=I&msg=" + Q(s));
}

CLICK_ON_ELEMENT = function(id){
    if (TOUCHES_DISABLED){
        LOG('touches disabled');
        // i think the click handle fires on the next page...
        return
    }
    if (!READY_FOR_USER){
        LOG('not ready for user yet');
        return;
    }
    if (BUTTONS[id]) BUTTONS[id]();
    else alert('no button for ' + id);

    prevent_double_touch();
}


_SHOWN_MODAL = undefined;
SHOW_MODAL = function(id){
    _SHOWN_MODAL = id;
    $("#" + id).modal({
                      "backdrop"  : "static",
                      "keyboard"  : true,
                      "show"      : true // ensure the modal is shown immediately
                      });
}

HIDE_MODAL = function(id){
    if (!id) return;
    $('#' + id).modal('hide');
    _SHOWN_MODAL = undefined;
};

sync_state_then_callback = function(callback, extraQuery){
    if (SYNC_DISABLED === true) return;

    var js_state = Q('{"counters":{"PASSES":' + GAMESTATE.PASSES
                     + ', "FREE_EPISODES":' + GAMESTATE.FREE_EPISODES
                     + '}}');

    // todo: invent a new name each time?
    did_sync = callback;
    if (extraQuery) {
        B("set/did_sync?json=" + js_state + extraQuery);
    } else {
        B("set/did_sync?json=" + js_state);
    }
}

did_sync = function(){};

next = function(from){
    unschedule_local_notif();
    // have to wait...
    setTimeout(function(){
               // tell the native code to advance to the next node in the story
               LOG('starting episode!');
               B('next');
               }, 1);
}

combine_app_and_server_info = function(){
    if (!GAMESTATE) return;
    STORYDATA = GAMESTATE.StoryInfo;

    // Have default values if not in a story
    GAMESTATE.AuthorName = STORYDATA.authorName || 'Author';
    GAMESTATE.AuthorBio = STORYDATA.authorBiography || '';
    GAMESTATE.IconURL = 'http://pg-story.appspot.com/dcon/' + STORYDATA.iconImage;
    if (STORYDATA.iconImage === undefined) GAMESTATE.IconURL = '';
    GAMESTATE.StoryName = STORYDATA.displayName;
    GAMESTATE.EndOfContentNote = STORYDATA.endOfContentNote || 'Thanks for playing!';

    update_view();
}

update_view = function(){
    // update the UI to match the GAMESTATE
    $('.num_passes').html(GAMESTATE.PASSES);

    // what is this?
    $('#counters').show();
    $('.ChapterNum').html(GAMESTATE.ChapterNum);
    $('#updates').hide();
    $('.StoryName').html(GAMESTATE.StoryName);
    $('#tagline').html(GAMESTATE.StoryInfo.tagline);
    $('#storyDescriptionBody').html(GAMESTATE.LandingInfo.storyDescription);
    $('.storyDescription').hide();

    $('#AuthorBio').html(GAMESTATE.AuthorBio);
    $('#AuthorName').html(GAMESTATE.AuthorName);
    $('#EndOfContentNote').html(GAMESTATE.EndOfContentNote);

    // remove the author thing for now
    $('.author').remove();

    $('#referral_code').html(GAMESTATE.ReferralCode);
    $('#friends_referred').html(GAMESTATE.ReferredFriends);
    $('#passes_earned').html(GAMESTATE.EarnedReferralPasses);

    var ss = GAMESTATE.LandingInfo.screenshots
    for (var i = 0; i < ss.length; ++i) {
        var img = $('<img class="screenshot" src=""/>').attr('src', 'https://pgnarrative.appspot.com/dcon/get/' + ss[i]);
        $('#screenshotHolder').append(img);
    }

    $('.icon').attr('src', GAMESTATE.IconURL);

    HIDE_MODAL(_SHOWN_MODAL);

    LOG('showing!');
    READY_FOR_USER = true;
};

// -------------------------------
// PART SIX: define click handlers for elements
// -------------------------------

BUTTONS = {};

BUTTONS.start_first_episode = function(){
    next();
}
BUTTONS.start = function(){
    window.location.href = 'episode://episode/'+GAMESTATE.StoryID;
}

BUTTONS.like = function(){
    LOG('tapped the like button!');
}

BUTTONS.share = function(){
    LOG('Yeah Mr. White! SHARING');
}

BUTTONS.alerts = function(){
    LOG('WAKEY WAKY');
}

BUTTONS.email = function(){
    // we need an in-app alert...
    // wire up the actual modal functionality and show the dialog
    SHOW_MODAL('ask_for_email');
}

BUTTONS.email_ok = function(){
    HIDE_MODAL('ask_for_email');
    LOG($('#user_email').val());
}

BUTTONS.show_console = function(){
    SHOW_MODAL('modal_console');
}

BUTTONS.dismiss_console = function(){
    HIDE_MODAL('modal_console');
}

BUTTONS.menu = function(){
    // quit to the catalog
    LOG('menu!');
    window.location.href = 'episode:///';
}

BUTTONS.play_another_story = function(){
    window.location.href = 'episode:///';
}

BUTTONS.contact = function(){
    var URL = 'http://pg-story.appspot.com/ask/' + GAMESTATE.StoryID + '/';
    URL += '?pgid=' + GAMESTATE.PGID;
    window.location.href = URL;
}

BUTTONS.faq = function(){
    var URL = 'http://pg-story.appspot.com/messages/' + GAMESTATE.StoryID + '/';
    URL += '?pgid=' + GAMESTATE.PGID;
    window.location.href = URL;
}

// Referral

BUTTONS.applyreferral = function(){
    SYNC_DISABLED = true;

    // get value from text field
    referral_code = $('#referral_input').val();
    query_referral_code = Q(referral_code)
    B('applyreferral?referralCode=' + query_referral_code);
}

did_applyreferral = function(result){
    if (result.PASSES != undefined) {
        GAMESTATE.PASSES = result.PASSES;
    }
    SYNC_DISABLED = false;

    if ('failure' === result.referralResult) {
        // iOS code already presents the error to the user; so, do nothing
    } else if ('success' === result.referralResult) {
        // switch to the version of this view which does not have the text box
        sync_state_then_callback(did_sync_referral_progress)
    } {
        // ???
    }
}

did_sync_referral_progress = function() {
    B('referral');
}

BUTTONS.referral = function(){
    B('referral');
}

BUTTONS.referral_popup = function(){
    B('referral?popup=true');
}

BUTTONS.sharereferral = function(){
    // Legally cannot provide a default message
    messageQuery = Q(' ');
    B('sharereferral?message=' + messageQuery);
}

did_sharereferral = function(){}

// Facebook
BUTTONS.login_fb = function(){
    B('fblogin');
}

did_fb_login = function(){}

BUTTONS.reloader = function() {
    location.reload(true);
};

did_reloader = function() {};

// -------------------------------
// PART SEVEN: Parameters
// -------------------------------

FREE_EPISODES_PER_DAY = 2;
STORYDATA = null; // fetched via JSONP
GAMESTATE = null;
SYNC_DISABLED = false;
